from magu.core.server import run_server
from magu.controllers.controller import *
from magu.models.model import *
from magu.repositories.repository import MariaDBRepository
from magu.database.database import MySQLDatabase as mysql

@table(name="tabela")
class RestModel(Model):
    def __init__(self):
        super().__init__()

    id = Column(int, id=True)
    name = Column(str)
    age = Column(int)

@request_mapping("/")
class RootController(Controller):
    def __init__(self):
        super().__init__()

    @get_mapping("/")
    def get_root(self):
        return {"root": "root"}


@request_mapping("/a")
class RestController(Controller):
    def __init__(self):
        super().__init__()

    @get_mapping()
    def get(self):
        return self.rep.find_all()

    @post_mapping()
    def post(self):
        mod = RestModel()
        mod.name = "delicia"
        mod.age = 1
        return self.rep.save(mod)

    @get_mapping("/{id}")
    def get_one(self):
        return self.rep.find_by_id(self.pk)

    @get_mapping("/a/{id}")
    def get_a(self):
        return self.rep.find_by_id(id)

    @put_mapping("/{id}")
    def patch(self):
        mod = RestModel()
        mod.name = "Enzo"
        mod.age = 17
        return self.rep.update(self.pk, mod)
    
cont = RestController()
root = RootController()

if __name__ == '__main__':
    run_server()



import os
import importlib.util

def load_properties():
    project_root = os.getcwd()
    
    prop_path = os.path.join(project_root, "properties.py")

    if not os.path.exists(prop_path):
        raise FileNotFoundError(
            f"[Magu] properties.py not found at {prop_path}"
        )

    spec = importlib.util.spec_from_file_location("user_properties", prop_path)
    module = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(module)
    return module

props = load_properties()

DATABASE_HOST = props.DATABASE_HOST
DATABASE_USER = props.DATABASE_USER
DATABASE_PASSWORD = props.DATABASE_PASSWORD
DATABASE_PORT=props.DATABASE_PORT
DATABASE_NAME = props.DATABASE_NAME

SERVER_HOST=props.SERVER_HOST
SERVER_PORT=props.SERVER_PORT
SERVER_ADDRESS=props.SERVER_ADDRESS

SECRET_KEY=props.SECRET_KEY



import mysql.connector
from mysql.connector import Error
from contextlib import contextmanager
#import mariadb
from magu import properties

class MySQLDatabase:
    @staticmethod
    def connect():
        try:
            conn = mysql.connector.connect(
                host=properties.DATABASE_HOST,
                user=properties.DATABASE_USER,
                password=properties.DATABASE_PASSWORD,
                port=properties.DATABASE_PORT,
                database=properties.DATABASE_NAME,
            )
            print(f"[DB] Connected with user {properties.DATABASE_USER} on host {properties.DATABASE_HOST}")
            conn.cursor().execute(f"USE {properties.DATABASE_NAME};")
            return conn
        except Error as e:
            print(f"[DB] ERROR: Couldn't connect to user {properties.DATABASE_USER} on host {properties.DATABASE_HOST}")
            return None
        
    @staticmethod
    @contextmanager
    def session():
        conn = MySQLDatabase.connect()
        if not conn or not conn.is_connected():
            raise ConnectionError("[DB] Couldn't create session")
        
        cursor = conn.cursor()
        try:
            yield cursor
            conn.commit()
        except Exception as e:
            conn.rollback()
            print(f"[DB] Query failed. Exception: {e}")
        finally:
            cursor.close()
            conn.close()

class MariaDBDatabase:
    @staticmethod
    def connect():
        try:
            conn = mariadb.connect(
                host=properties.DATABASE_HOST,
                user=properties.DATABASE_USER,
                password=properties.DATABASE_PASSWORD,
                port=properties.DATABASE_PORT,
                database=properties.DATABASE_NAME,
            )
            print(f"[DB] Connected with user {properties.DATABASE_USER} on host {properties.DATABASE_HOST}")
            conn.cursor().execute(f"USE `{properties.DATABASE_NAME}`;")
            return conn
        except Error as e:
            print(f"[DB] ERROR: Couldn't connect to user {properties.DATABASE_USER} on host {properties.DATABASE_HOST}")
            return None
        
    @staticmethod
    @contextmanager
    def session():
        conn = MariaDBDatabase.connect()
        if not conn:
            raise ConnectionError("[DB] Couldn't create session")
        
        cursor = conn.cursor()
        try:
            yield cursor
            conn.commit()
        except Exception as e:
            conn.rollback()
            print("[DB] Query failed")
        finally:
            cursor.close()
            conn.close()



from http.server import SimpleHTTPRequestHandler
import json
from urllib.parse import urlsplit, unquote

class Handler(SimpleHTTPRequestHandler):
    def send_json(self, data: dict, status: int = 200):
        self.send_response(status)
        self.send_header("Content-Type", "application/json")
        self.end_headers()
        self.wfile.write(json.dumps(data).encode('utf-8'))

    def send_400_status(self, message: str, status: int = 400):
        self.send_response(status)
        self.end_headers()
        self.wfile.write(b"404 Not Found")

    def send_token(self, data, token, status: int = 200):
        self.send_response(status)
        self.send_header("Content-Type", "application/json")
        self.send_header("Authorization", f"Bearer {token}")
        self.end_headers()
        self.wfile.write(json.dumps(data).encode("utf-8"))

    def parse_json_body(self):
        content_length = int(self.headers.get("Content-Length", 0))
        body = self.rfile.read(content_length).decode("utf-8")
        try:
            return json.loads(body)
        except json.JSONDecodeError:
            self.send_error(400, "Bad Request")
    
    def parse_headers(self):
        auth_header = self.headers.get("Authorization")
        if auth_header and auth_header.startswith("Bearer "):
            return auth_header.split(" ")[1]
        return None
    
    def parse_path(self, url):
        split = urlsplit(url)
        path_parts = split.path.strip("/").split("/")
        result = {"path": "/" + "/".join(path_parts)}

        if path_parts and path_parts[-1].isdigit():
            result["id"] = int(path_parts[-1])
        else:
            result["id"] = False

        result["query"] = unquote(split.query.strip().lower())

        return result



from http.server import HTTPServer
from magu.services.handler import Handler
from magu.core.router import Router
import magu.properties as properties
import json

class ServerHandler(Handler):
    router = Router()

    def do_GET(self):
        self.router.route(self, 'GET')
    
    def do_POST(self):
        length = int(self.headers.get("Content-Length", 0))
        if length:
            body = self.rfile.read(length)
            data = json.loads(body)

        self.router.route(self, 'POST', data=data)

    def do_PUT(self):
        length = int(self.headers.get("Content-Length", 0))
        if length:
            body = self.rfile.read(length)
            data = json.loads(body)

        self.router.route(self, 'PUT', data=data)
    
    def do_PATCH(self):
        length = int(self.headers.get("Content-Length", 0))
        if length:
            body = self.rfile.read(length)
            data = json.loads(body)

        self.router.route(self,'PATCH', data=data)

    def do_DELETE(self):
        self.router.route(self,'DELETE')
        

def run_server():
    httpd = HTTPServer((properties.SERVER_HOST, properties.SERVER_PORT), ServerHandler)
    print(f"[Server] Server running on {properties.SERVER_ADDRESS}")
    httpd.serve_forever()



from magu.controllers.controller import Controller
from magu.services.handler import Handler
import inspect
import re

routes: dict[str, tuple] = {}

def get_routes(controller: Controller.__class__):
     for _, func in inspect.getmembers(controller, inspect.isfunction):
          if hasattr(func, "_http_method"):
            if func._method_uri.find("{") != -1 and func._method_uri.find("}") != -1:
                route = f"{func._http_method} /{controller._uri.replace("/", "")}{func._method_uri}"
            elif func._method_uri.strip() != "/":
                route = f"{func._http_method} /{controller._uri.replace("/", "")}/{func._method_uri.replace("/", "")}"
            else:
                route = f"{func._http_method} /{controller._uri.replace("/", "")}"
            routes[route] = (func, controller)


def get_methods(uri: str, method: str, id: str = None):
        methods = []
        for name in routes:
            route_method = name.split(" ")[0]
            route_uri = name.split(" ")[1]

            if route_uri.find("{") != -1 and route_uri.find("}") != -1:
                route_uri = re.sub(r"({(.*?)})", id, route_uri)

            controller = routes[name]
            if route_method.strip() == method and route_uri.strip() == uri:
                methods.append((routes[name], controller))
             
        return methods


class Router:
    def route(self, handler: Handler, http_method: str, data: dict = {}):
        server_path = handler.path
        parse_path = handler.parse_path(server_path)
        
        pk = re.findall(r"(?<=/)\d+", parse_path["path"])[0] if re.findall(r"(?<=/)\d+", parse_path["path"]) else ""

        try:
            gets = get_methods(parse_path["path"], http_method, pk)[0]
        except IndexError:
            handler.send_json({"404": "Not Found"}, status=404)
            return

        for func, cont in gets:
            cont.pk = pk
            cont.data = data
            instance = cont()
            handler.send_json(func(instance))
            break



import jwt
from datetime import datetime, timedelta
from models.model import Model
import magu.properties as properties

class Authentication:
    @staticmethod
    def get_token(user: Model):
        if not user._user:
            raise Exception("[Auth] The provided model is not a user")
        
        payload: dict = user.__dict__
        payload["exp"] = datetime.now() + timedelta(hours=1)

        return jwt.encode(payload, properties.SECRET_KEY, algorithm="HS256")
    
    @staticmethod 
    def verify_token(token: str):
        try:
            return jwt.decode(token, properties.SECRET_KEY, algorithms=["HS256"])
        except jwt.InvalidTokenError:
            raise Exception("[Auth] Invalid token")
        except jwt.ExpiredSignatureError:
            raise Exception("[Auth] Token expired")
        
    @staticmethod
    def verify_password(password: str, db_password: str):
        return password.strip() == db_password.strip()
